/*fe14cs compute collision strengths for forbidden transitions 
 * w/in Fe XIV ground term. From 
 * >>refer Storey, P.J., Mason, H.E., Saraph, H.E., 1996, A&A, 309, 677
 * */
#include "cddefines.h"
#include "fe14cs.h"

void fe14cs(double te1, 
	  double *csfe14)
{
	double a, 
	  b, 
	  c, 
	  d, 
	  telog1, 
	  telog2, 
	  telog3;

#	ifdef DEBUG_FUN
	fputs( "<+>fe14cs()\n", debug_fp );
#	endif

	/* limit range in log T: */
	telog1 = te1;
	telog1 = MIN2(9.0,telog1);
	telog1 = MAX2(4.0,telog1);

	/* compute square and cube */
	telog2 = telog1*telog1;
	telog3 = telog2*telog1;

	/* compute cs:
	 * */
	if( telog1 <= 5.0 )
	{
		a = 557.05536;
		b = -324.56109;
		c = 63.437974;
		d = -4.1365147;
		*csfe14 = a + b*telog1 + c*telog2 + d*telog3;
	}
	else
	{
		a = 0.19515493;
		b = 2.9404407;
		c = 4.9578944;
		d = 0.79887506;
		*csfe14 = a + b*exp(-0.5*((telog1-c)*(telog1-c)/d));
	}

#	ifdef DEBUG_FUN
	fputs( " <->fe14cs()\n", debug_fp );
#	endif
	return;
}

